{% extends "!layout.html" %}
{% block footer %}
	{{ super() }}
	
	<script type="text/javascript">

	  var _gaq = _gaq || [];
	  _gaq.push(['_setAccount', 'UA-155626-6']);
	  _gaq.push(['_trackPageview']);

	  (function() {
	    var ga = document.createElement('script'); ga.type = 'text/javascript'; ga.async = true;
	    ga.src = ('https:' == document.location.protocol ? 'https://ssl' : 'http://www') + '.google-analytics.com/ga.js';
	    var s = document.getElementsByTagName('script')[0]; s.parentNode.insertBefore(ga, s);
	  })();

	</script>

{% endblock %}
{% block extrahead %}
	{{ super() }}


   <style type="text/css">
	body { 
		font-family: Verdana, sans-serif;
		font-size: 16px;
	}

	div.col1 {
		display: block;
		float: left;
	}

	div.col2 {
		margin-left: 3em;
		display: block;
		float: left;
	}

	div.after {
		clear: left;
	}
	
</style>

<link  href="http://fonts.googleapis.com/css?family=Lobster" rel="stylesheet" type="text/css" >
<link  href="http://fonts.googleapis.com/css?family=Droid+Serif" rel="stylesheet" type="text/css" >
<link  href="static/procgraph.css" rel="stylesheet" type="text/css" >


{% endblock %}
{% block copyright %}
{% endblock %}
