from .debug_components import *

# TODO: make this list automatically generated


def load(x):
    __import__(x)


load("procgraph_cv")
# import procgraph_cv
load("procgraph_foo")
# import procgraph_foo
load("procgraph_hdf")
# import procgraph_hdf
load("procgraph_images")
# import procgraph_images
load("procgraph_io_misc")
# import procgraph_io_misc
load("procgraph_mpl")
# import procgraph_mpl
load("procgraph_mplayer")
# import procgraph_mplayer
load("procgraph_numpy_ops")
# import procgraph_numpy_ops
load("procgraph_pil")
# import procgraph_pil
load("procgraph_robotics")
# import procgraph_robotics
load("procgraph_signals")
# import procgraph_signals
load("procgraph_statistics")
# import procgraph_statistics
