# Autogenerated file -- do not modify.

# This file is generated by "pgindex".
# It lists the default ProcGraph packages and their dependencies
# so that we can warn the user if some dependency is not found at
# install time. ProcGraph's policy is to be very liberal; for example,
# the `procgraph_cv' package is installed even though OpenCV is not found.
# and an error will be thrown only if the user actually uses those blocks.

index = {
    "packages": {
        "procgraph.components.debug_components": {
            "blocks": ["info", "constant", "+", "*", "-", "/", "gain", "print", "identity"],
            "desc": "Components used for debugging and unit tests.",
            "requires": {},
        },
        "procgraph_cv": {
            "blocks": ["gradient", "smooth"],
            "desc": "Blocks using the OpenCV library.",
            "requires": {"cv": ["cv", "opencv"]},
        },
        "procgraph_foo": {
            "blocks": ["block_example"],
            "desc": "An example package for ProcGraph that shows how to organize your code. ",
            "requires": {"pickle": ["pickle"]},
        },
        "procgraph_hdf": {
            "blocks": ["hdfread_many", "hdfread_test", "hdfread_many_test", "hdfwrite", "hdfread"],
            "desc": "This is a set of blocks to read and write logs in HDF_ format. ",
            "requires": {"tables": ["tables"]},
        },
        "procgraph_images": {
            "blocks": [
                "rgb2gray",
                "compose",
                "reshape2d",
                "solid",
                "blend",
                "skim_top_and_bottom",
                "posterize",
                "gray2rgb",
                "scale",
                "grid",
                "skim_top",
                "posneg",
                "border",
            ],
            "desc": "Blocks for basic operations on images. ",
            "requires": {},
        },
        "procgraph_io_misc": {
            "blocks": ["pickle_load", "to_file", "pickle", "pickle_group", "as_json"],
            "desc": "Miscellaneous functions to be better organized.",
            "requires": {"json": ["simplejson"]},
        },
        "procgraph_mpl": {
            "blocks": ["plot"],
            "desc": "Blocks using Matplotlib to display data.",
            "requires": {"matplotlib": ["matplotlib"], "matplotlib.pylab": ["matplotlib.pylab"]},
        },
        "procgraph_mplayer": {
            "blocks": ["mplayer", "mencoder"],
            "desc": "Blocks for encoding/decoding video based on MPlayer.",
            "requires": {},
        },
        "procgraph_numpy_ops": {
            "blocks": [
                "smooth1d",
                "square",
                "sign",
                "minimum",
                "select",
                "rad2deg",
                "outer",
                "log",
                "sum",
                "astype",
                "arctan",
                "abs",
                "take",
                "real",
                "deg2rad",
                "hstack",
                "flipud",
                "max",
                "vstack",
                "gradient1d",
                "dstack",
                "maximum",
                "fliplr",
                "normalize_Linf",
                "mean",
            ],
            "desc": "Various operations wrapping numpy functions.",
            "requires": {},
        },
        "procgraph_pil": {
            "blocks": ["imread", "text", "resize"],
            "desc": "Blocks for image operations based on the PIL library",
            "requires": {"PIL": ["PIL"]},
        },
        "procgraph_robotics": {
            "blocks": [
                "pose2commands",
                "laser_display",
                "skim",
                "organic_scale",
                "pose2vel_",
                "laser_dot_display",
            ],
            "desc": "Some functions specific to robotics applications. ",
            "requires": {"geometry": ["geometry"]},
        },
        "procgraph_ros": {
            "blocks": ["bagread_test", "ros2python", "bagread"],
            "desc": "This is a set of blocks to read and write logs in ROS_ Bag format. ",
            "requires": {"ros": ["ros"]},
        },
        "procgraph_signals": {
            "blocks": [
                "derivative2",
                "slice",
                "two_step_difference",
                "derivative",
                "join",
                "sync",
                "fps_data_limit",
                "history",
                "historyt",
                "forward_difference",
                "low_pass",
                "make_tuple",
                "fps_limit",
                "fps_print",
                "last_n_samples",
                "sieve",
                "extract",
                "any",
                "wait",
            ],
            "desc": "Blocks performing operations with a dynamic nature. ",
            "requires": {},
        },
        "procgraph_statistics": {
            "blocks": ["cov2corr", "normalize", "expectation", "covariance", "soft_variance", "variance"],
            "desc": "Blocks for common statistical operations.",
            "requires": {},
        },
        "procgraph_yaml": {
            "blocks": ["yaml2object"],
            "desc": "YAML conversions.",
            "requires": {"yaml": ["yaml"]},
        },
    }
}
