/* .procgraph-block P, .procgraph-block LI { max-width: 40em;} */
.procgraph-module h1 { 
           font-size: 150%;
           margin-left: -3em;
        background-color: #ddd; padding: 10px;
}
.procgraph-module h1 tt {
       font-weight: bold; 
       font-size: 120%;
}

.procgraph-block h2 { background-color: #ddd; padding: 5px;}
.procgraph-block h2 tt { font-weight: bold; font-size: 120%;}

.procgraph-output, .procgraph-input, .procgraph-config { 
    clear: left; 
    display: block; 
    overflow: auto;
}
.procgraph-output ul, .procgraph-input ul, .procgraph-config ul { 
    overflow: auto;
}
.procgraph-output ul li, .procgraph-input ul li, .procgraph-config ul li {
    display: block;
}

.procgraph-output h3, .procgraph-input h3, .procgraph-config h3 { 
    display: block;  
    float: left;
    padding: 0; margin: 0;
    min-width: 12em;
    text-align: right;
}
.procgraph-source, .procgraph-source A { 
    clear: left;    
    color: gray; 
}

table.docutils { width: 100%;}


